#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________


__all__ = ['TestCase']

import os
import pyutilib.th
import copy
import pyutilib.subprocess
from coopr.opt import SolverResults
import re
import pyutilib.misc


def _failIfCooprResultsDiffer(self, cmd=None, baseline=None, cwd=None):
    if cwd is not None:
        oldpwd = os.getcwd()
        os.chdir(cwd)
    output = pyutilib.subprocess.run(cmd)
    if output[0] != 0:
        self.fail("Command terminated with nonzero status: '%s'" % cmd)
    results = extract_results( re.split('\n',output[1]) )
    if os.path.exists(baseline):
        INPUT = open(baseline, 'r')
        baseline = "\n".join(INPUT.readlines())
        INPUT.close()
    try:
        compare_results(results, baseline)
    except IOError, err:
        self.fail("Command failed to generate results that can be compared with the baseline: '%s'" % str(err))
    except ValueError, err:
        self.fail("Difference between results and baseline: '%s'" % str(err))


class TestCase(pyutilib.th.TestCase):

    def __init__(self, methodName='runTest'):
        pyutilib.th.TestCase.__init__(self, methodName)

    def failIfCooprResultsDiffer(self, cmd, baseline, cwd=None):
        if not using_yaml:
            self.fail("Cannot compare Coopr results because PyYaml is not installed")
        _failIfCooprresultsDiffer(self, cmd=cmd, baseline=baseline, cwd=cwd)

    def add_coopr_results_test(name=None, cmd=None, fn=None, baseline=None):
        if not using_yaml:
            return
        if cmd is None and fn is None:
            print "ERROR: must specify either the 'cmd' or 'fn' option to define how the output file is generated"
            return
        if name is None and baseline is None:
            print "ERROR: must specify a baseline comparison file, or the test name"
            return
        if baseline is None:
            baseline=name+".txt"
        tmp = name.replace("/","_")
        tmp = tmp.replace("\\","_")
        tmp = tmp.replace(".","_")
        #
        # Create an explicit function so we can assign it a __name__ attribute.
        # This is needed by the 'nose' package
        #
        if fn is None:
            currdir=os.getcwd()
            func = lambda self,c1=currdir,c2=cmd,c3=tmp+".out",c4=baseline: _failIfCooprResultsDiffer(self,cwd=c1,cmd=c2,baseline=c4)
        else:
            # This option isn't implemented...
            sys.exit(1)
            func = lambda self,c1=fn,c2=tmp,c3=baseline: _failIfCooprResultsDiffer(self,fn=c1,name=c2,baseline=c3)
        func.__name__ = "test_"+tmp
        setattr(TestCase, "test_"+tmp, func)
    add_coopr_results_test=staticmethod(add_coopr_results_test)






#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['ILMLicensedSystemCallSolver']

import shellcmd
import pyutilib.services
import pyutilib.subprocess
import pyutilib.common
import re
import sys
import os

class ILMLicensedSystemCallSolver(shellcmd.SystemCallSolver):
    """ A command line solver that launches executables licensed with ILM """

    def __init__(self, **kwds):
        """ Constructor """
        shellcmd.SystemCallSolver.__init__(self, **kwds)

    def available(self, exception_flag=True):
        """ True if the solver is available """
        if self._assert_available:
            return True
        if not shellcmd.SystemCallSolver.available(self, exception_flag):
            return False
        executable = pyutilib.services.registered_executable("ilmlist")
        if not executable is None:
            try:
                [rc,log] = pyutilib.subprocess.run(executable.get_path())
            except pyutilib.common.WindowsError, err:
                raise pyutilib.common.ApplicationError, "Could not execute the command: ilmtest\n\tError message: "+   str(err)
            sys.stdout.flush()
            for line in log.split("\n"):
                tokens = re.split('[\t ]+',line.strip())
                if len(tokens) == 5 and tokens[0] == 'tokens' and tokens[1] == 'reserved:' and tokens[4] == os.environ.get('USER',None):
                    if not (tokens[2] == 'none' or tokens[2] == '0'):
                        return True
                    return False
                elif len(tokens) == 3 and tokens[0] == 'available' and tokens[1] == 'tokens:':
                    if tokens[2] == '0':
                        return False
                    break
                elif len(tokens) == 6 and tokens[1] == 'server' and tokens[5] == 'DOWN.':
                    return False
        return True

pyutilib.services.register_executable(name="ilmlist")



#
# Unit Tests for coopr.opt.base.solution
#
#

import os
import sys
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__)))+"/../..")
cooprdir = dirname(abspath(__file__))+os.sep+".."+os.sep+".."+os.sep
currdir = dirname(abspath(__file__))+os.sep

from nose.tools import nottest
import coopr.opt
import coopr
import pyutilib.th as unittest
import pyutilib.misc
import pyutilib.services
import xml
import pickle

try:
    import yaml
    yaml_available=True
except ImportError:
    yaml_available=False


class Test(unittest.TestCase):

    def setUp(self):
        pyutilib.services.TempfileManager.tempdir = currdir
        self.results = coopr.opt.SolverResults()
        self.soln = self.results.solution.add()
        self.soln.variable[1]=0.0
        self.soln.variable[2]=0.0
        self.soln.variable[4]=0.0
        pyutilib.services.TempfileManager.clear_tempfiles()

    def tearDown(self):
        del self.results

    def test_write_solution1(self):
        """ Write a SolverResults Object with solutions """
        self.results.write(filename=currdir+"write_solution1.txt")
        if not os.path.exists(currdir+"write_solution1.txt"):
           self.fail("test_write_solution - failed to write write_solution1.txt")
        self.failUnlessFileEqualsBaseline(currdir+"write_solution1.txt", currdir+"test1_soln.txt")

    def test_write_solution2(self):
        """ Write a SolverResults Object without solutions """
        self.results.write(num=None,filename=currdir+"write_solution2.txt")
        if not os.path.exists(currdir+"write_solution2.txt"):
           self.fail("test_write_solution - failed to write write_solution2.txt")
        self.failUnlessFileEqualsBaseline(currdir+"write_solution2.txt", currdir+"test2_soln.txt")

    @unittest.skipIf(not yaml_available, "Cannot import 'yaml'")
    def test_read_solution1(self):
        """ Read a SolverResults Object"""
        self.results = coopr.opt.SolverResults()
        self.results.read(filename=currdir+"test4_sol.txt")
        self.results.write(filename=currdir+"read_solution1.out")
        if not os.path.exists(currdir+"read_solution1.out"):
           self.fail("test_read_solution1 - failed to write read_solution1.out")
        self.failUnlessFileEqualsBaseline(currdir+"read_solution1.out", currdir+"test4_sol.txt")

    @unittest.skipIf(not yaml_available, "Cannot import 'yaml'")
    def test_pickle_solution1(self):
        """ Read a SolverResults Object"""
        self.results = coopr.opt.SolverResults()
        self.results.read(filename=currdir+"test4_sol.txt")
        str = pickle.dumps(self.results)
        res = pickle.loads(str)
        self.results.write(filename=currdir+"read_solution1.out")
        if not os.path.exists(currdir+"read_solution1.out"):
           self.fail("test_read_solution1 - failed to write read_solution1.out")
        self.failUnlessFileEqualsBaseline(currdir+"read_solution1.out", currdir+"test4_sol.txt")

    #
    # deleting is not supported right now
    #
    def Xtest_delete_solution(self):
        """ Delete a solution from a SolverResults object """
        self.results.solution.delete(0)
        self.results.write(filename=currdir+"delete_solution.txt")
        if not os.path.exists(currdir+"delete_solution.txt"):
           self.fail("test_write_solution - failed to write delete_solution.txt")
        self.failUnlessFileEqualsBaseline(currdir+"delete_solution.txt", currdir+"test4_soln.txt")

    def test_get_solution(self):
        """ Get a solution from a SolverResults object """
        tmp = self.results.solution[0]
        self.failUnlessEqual(tmp,self.soln)

    def test_get_solution_attr_error(self):
        """ Create an error with a solution suffix """
        try:
           tmp = self.soln.bad
           self.fail("Expected attribute error failure for 'bad'")
        except AttributeError:
           pass

    #
    # This is currently allowed, although soln.variable = True is equivalent to
    #   soln.variable.value = True
    #
    def Xtest_set_solution_attr_error(self):
        """ Create an error with a solution suffix """
        try:
           self.soln.variable = True
           self.fail("Expected attribute error failure for 'variable'")
        except AttributeError:
           pass

    def test_soln_pprint1(self):
        """ Write a solution with only zero values, using the results 'write()' method """
        self.soln.variable[1]=0.0
        self.soln.variable[2]=0.0
        self.soln.variable[4]=0.0
        self.results.write(filename=currdir+"soln_pprint.txt")
        if not os.path.exists(currdir+"soln_pprint.txt"):
           self.fail("test_write_solution - failed to write soln_pprint.txt")
        self.failUnlessFileEqualsBaseline(currdir+"soln_pprint.txt", currdir+"test3_soln.txt")

    def test_soln_pprint2(self):
        """ Write a solution with only zero values, using the Solution.pprint() method """
        self.soln.variable[1]=0.0
        self.soln.variable[2]=0.0
        self.soln.variable[4]=0.0
        pyutilib.misc.setup_redirect(currdir+"soln_pprint2.out")
        print self.soln
        print self.soln.variable
        pyutilib.misc.reset_redirect()
        self.failUnlessFileEqualsBaseline(currdir+"soln_pprint2.out", currdir+"soln_pprint2.txt")

    def test_soln_suffix_iter(self):
        """ Test a suffix iterator """
        self.soln.variable[1]=0.0
        self.soln.variable[2]=0.1
        self.soln.variable[4]=0.3
        i=0
        for key in self.soln.variable:
          i=i+1
          self.failUnlessEqual(self.soln.variable[key].value, self.soln.variable[key].id/10.0)
        self.failUnlessEqual(i,len(self.soln.variable))

    def test_soln_suffix_getiter(self):
        self.soln.variable["x0"]=0.0
        self.soln.variable[2]=0.1
        self.soln.variable["x3"]=0.3
        self.failUnlessEqual(self.soln.variable["x3"].value,0.3)
        #print "HERE",self.soln.variable._names
        self.failUnlessEqual(self.soln.variable[2].value,0.1)

    def test_soln_suffix_setattr(self):
        self.soln.variable["x(0)"] = 0.0
        self.soln.variable["x3"]=0.3
        self.soln.variable["x3"].slack=0.4
        self.soln.variable["y[1,ab]"]=0.5
        #self.soln.variable.z[0,'a']=2.0
        #print "HERE",self.soln.variable._names
        #print "HERE",self.soln.variable.__dict__.keys()
        #print "HERE",self.soln.variable.keys()
        #print "HERE",self.soln.variable.__getitem__('y[1,ab]')
        #print "HERE",repr(self.soln.variable.x)
        #print "HERE",repr(self.soln.variable.x[0])
        #print "HERE",repr(self.soln.variable.y)
        #print "HERE",repr(self.soln.variable.y[1,'ab'])
        #print "HERE",repr(self.soln.variable.z)
        #print "HERE",repr(self.soln.variable.z[0,'a'])
        #print "HERE",repr(self.soln.variable.foo('x(0)'))
        #self.soln.pprint()
        #print "HERE", self.soln.variable._index.keys()
        self.failUnlessEqual(self.soln.variable.x.keys(),[0])
        self.failUnlessEqual(self.soln.variable.x[0].value,0.0)
        self.failUnlessEqual(self.soln.variable.x3.value,0.3)
        self.failUnlessEqual(self.soln.variable.x3.slack,0.4)
        self.failUnlessEqual(self.soln.variable.y.keys(),[(1,'ab')])
        self.failUnlessEqual(self.soln.variable.y[1,'ab'].value,0.5)


if __name__ == "__main__":
    import pyutilib.misc
    #sys.settrace(pyutilib.misc.traceit)
    unittest.main()



#
# Unit Tests for coopr.opt.base.OS
#
#

import os
import sys
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__)))+"/../..")
cooprdir = dirname(abspath(__file__))+os.sep+".."+os.sep+".."+os.sep
currdir = dirname(abspath(__file__))+os.sep

from nose.tools import nottest
import coopr.opt
import coopr
import xml
import pyutilib.th as unittest
import pyutilib.services


class Test(unittest.TestCase):

    def setUp(self):
        pyutilib.services.TempfileManager.tempdir = currdir

    def tearDown(self):
        pyutilib.services.TempfileManager.clear_tempfiles()
        if os.path.exists(currdir+"test_sol.txt"):
           os.remove(currdir+"test_sol.txt")

    def test_read_solution1(self):
        reader = coopr.opt.reader.sol.ResultsReader_sol()

    def test_factory(self):
        reader = coopr.opt.ReaderFactory("sol")
        soln = reader(currdir+"test4_sol.sol")
        soln.write(filename=currdir+"factory.txt")
        self.failUnlessFileEqualsBaseline(currdir+"factory.txt", currdir+"test4_sol.txt")
        

if __name__ == "__main__":
   unittest.main()



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

#
# Class for reading results with the SolverResults object
#

import os
import sys
import re

from coopr.opt.base import results
from coopr.opt.base.formats import *
from coopr.opt import SolverResults
from pyutilib.component.core import alias


class ResultsReader_results(results.AbstractResultsReader):
    """
    Class that reads in a *.results file and generates a 
    SolverResults object.
    """

    alias(str(ResultsFormat.yaml))

    def __init__(self):
        results.AbstractResultsReader.__init__(self,ResultsFormat.yaml)

    def __call__(self, filename, res=None, soln=None):
        """
        Parse a *.results file
        """
        if res is None:
            res = SolverResults()  
        #
        res.read(filename)
        return res




#
# Unit Tests for util/misc
#
#

import os
import sys
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__)))+"/../..")
cooprdir = dirname(abspath(__file__))+"/../.."
currdir = dirname(abspath(__file__))+os.sep

import unittest
from nose.tools import nottest
import coopr.opt
import coopr
import pyutilib.services
import pyutilib.component.core


class TestWriter(coopr.opt.AbstractProblemWriter):

    pyutilib.component.core.alias('wtest')

    def __init__(self, name=None):
        coopr.opt.AbstractProblemWriter.__init__(self,name)
    

class TestReader(coopr.opt.AbstractResultsReader):

    pyutilib.component.core.alias('rtest')

    def __init__(self, name=None):
        coopr.opt.AbstractResultsReader.__init__(self,name)
    

class TestSolver(coopr.opt.OptSolver):

    pyutilib.component.core.alias('stest')

    def __init__(self, **kwds):
        kwds['type'] = 'stest_type'
        kwds['doc'] = 'TestSolver Documentation'
        coopr.opt.OptSolver.__init__(self,**kwds)

    def enabled(self):
        return False


class Test(unittest.TestCase):

    def run(self, result=None):
        unittest.TestCase.run(self,result)

    def setUp(self):
        coopr.opt.SolverFactory.activate('stest')
        coopr.opt.WriterFactory.activate('wtest')
        coopr.opt.ReaderFactory.activate('rtest')
        pyutilib.services.TempfileManager.tempdir = currdir

    def tearDown(self):
        pyutilib.services.TempfileManager.clear_tempfiles()
        coopr.opt.SolverFactory.deactivate('stest')
        coopr.opt.WriterFactory.deactivate('wtest')
        coopr.opt.ReaderFactory.deactivate('rtest')

    def test_solver_factory(self):
        """
        Testing the coopr.opt solver factory
        """
        ans = sorted(coopr.opt.SolverFactory.services())
        #self.failUnlessEqual(len(ans),8)
        self.failUnless(set(['stest']) <= set(ans))

    def test_solver_instance(self):
        """
        Testing that we get a specific solver instance
        """
        ans = coopr.opt.SolverFactory("none")
        self.failUnlessEqual(ans, None)
        ans = coopr.opt.SolverFactory("stest")
        self.failUnlessEqual(type(ans), TestSolver)
        ans = coopr.opt.SolverFactory("stest", name="mymock")
        self.failUnlessEqual(type(ans), TestSolver)
        self.failUnlessEqual(ans.name,  "mymock")

    def test_solver_registration(self):
        """
        Testing methods in the solverwriter factory registration process
        """
        coopr.opt.SolverFactory.deactivate("stest")
        self.failUnless(not 'stest' in coopr.opt.SolverFactory.services())
        coopr.opt.SolverFactory.activate("stest")
        self.failUnless('stest' in coopr.opt.SolverFactory.services())

    def test_writer_factory(self):
        """
        Testing the coopr.opt writer factory
        """
        factory = coopr.opt.WriterFactory.services()
        self.failUnless(set(['wtest']) <= set(factory))

    def test_writer_instance(self):
        """
        Testing that we get a specific writer instance

        Note: this simply provides code coverage right now, but
        later it should be adapted to generate a specific writer.
        """
        ans = coopr.opt.WriterFactory("none")
        self.failUnlessEqual(ans, None)
        ans = coopr.opt.WriterFactory("wtest")
        self.failIfEqual(ans, None)
        #ans = coopr.opt.WriterFactory("wtest", "mywriter")
        #self.failIfEqual(ans, None)
        #self.failIfEqual(ans.name, "mywriter")

    def test_writer_registration(self):
        """
        Testing methods in the writer factory registration process
        """
        coopr.opt.WriterFactory.deactivate("wtest")
        self.failUnless(not 'wtest' in coopr.opt.WriterFactory.services())
        coopr.opt.WriterFactory.activate("wtest")
        self.failUnless('wtest' in coopr.opt.WriterFactory.services())


    def test_reader_factory(self):
        """
        Testing the coopr.opt reader factory
        """
        ans = coopr.opt.ReaderFactory.services()
        self.failUnlessEqual(len(ans),4)
        self.failUnlessEqual(set(ans), set(["osrl","rtest", "sol","yaml"]))

    def test_reader_instance(self):
        """
        Testing that we get a specific reader instance
        """
        ans = coopr.opt.ReaderFactory("none")
        self.failUnlessEqual(ans, None)
        ans = coopr.opt.ReaderFactory("osrl")
        self.failUnlessEqual(type(ans), coopr.opt.reader.OS.ResultsReader_osrl)
        #ans = coopr.opt.ReaderFactory("osrl", "myreader")
        #self.failUnlessEqual(type(ans), coopr.opt.reader.OS.ResultsReader_osrl)
        #self.failUnlessEqual(ans.name, "myreader")

    def test_reader_registration(self):
        """
        Testing methods in the reader factory registration process
        """
        coopr.opt.ReaderFactory.deactivate("rtest")
        self.failUnless(not 'rtest' in coopr.opt.ReaderFactory.services())
        coopr.opt.ReaderFactory.activate("rtest")
        self.failUnless('rtest' in coopr.opt.ReaderFactory.services())

if __name__ == "__main__":
   unittest.main()




#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

import pyutilib.component.core

pyutilib.component.core.PluginGlobals.push_env( 'coopr.opt' )

from base import *
from results import *
import solver
import reader
#import testing
from parallel import *

pyutilib.component.core.PluginGlobals.pop_env()


try:
    import pkg_resources
    #
    # Load modules associated with Plugins that are defined in 
    # EGG files.
    #
    for entrypoint in pkg_resources.iter_entry_points('coopr.opt'):
        plugin_class = entrypoint.load()
except:
    pass




#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

#
# The formats that are supported by Pyomo
#
__all__ = ['ProblemFormat', 'ResultsFormat', 'guess_format']

from pyutilib.enum import Enum

#
# pyomo - A coopr.pyomo.PyomoModel object, or a *.py file that defines such an object
# cpxlp - A CPLEX LP file
# nl - AMPL *.nl file
# mps - A free-format MPS file
# mod - AMPL *.mod file
# lpxlp - A LPSolve LP file
# osil - An XML file defined by the COIN-OR OS project: Instance
# colin - A COLIN shell command
# colin_optproblem - A Python object that inherits from
#                   coopr.opt.colin.OptProblem (this can wrap a COLIN shell
#                   command, or provide a runtime optimization problem)
#
ProblemFormat = Enum('colin', 'pyomo', 'cpxlp', 'nl', 'mps', 'mod', 'lpxlp', 'osil', 'colin_optproblem', )

#
# osrl - osrl XML file defined by the COIN-OR OS project: Result
# results - A Pyomo results object  (reader define by solver class)
# sol - AMPL *.sol file
# soln - A solver-specific solution file  (reader define by solver class)
# yaml - A Pyomo results file in YAML format
#
ResultsFormat = Enum('osrl', 'results', 'sol', 'soln', 'yaml')


def guess_format(filename):
    formats = {}
    formats['py']=ProblemFormat.pyomo
    formats['nl']=ProblemFormat.nl
    formats['mps']=ProblemFormat.mps
    formats['mod']=ProblemFormat.mod
    formats['lp']=ProblemFormat.cpxlp
    formats['sol']=ResultsFormat.sol
    formats['osrl']=ResultsFormat.osrl
    formats['soln']=ResultsFormat.soln
    formats['yml']=ResultsFormat.yaml
    formats['results']=ResultsFormat.yaml
    for fmt in formats:
        if filename.endswith('.'+fmt):
            return formats[fmt]
    return None




#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['SolverResults', 'SolverStatus', 'ProblemSense', 'SolutionStatus', 'Solution']

from pyutilib.misc import format_io
from pyutilib.math import as_number
from pyutilib.enum import Enum

import sys
import StringIO

SolverStatus = Enum('error', 'warning', 'ok', 'aborted')
ProblemSense = Enum('unknown', 'minimize', 'maximize')
SolutionStatus = Enum('unbounded', 'globallyOptimal', 'locallyOptimal',
                            'optimal', 'bestSoFar', 'feasible', 'infeasible',
                            'stoppedByLimit', 'unsure', 'error', 'other')


class _SolutionSuffix(object):

    def __init__(self, description):
        self._info = []
        self._index = {}
        self._count = {}
        self.description = description

    def add(self, index, value):
        """ NOTE: this function is deprecated """
        self.__setitem__(index, value)

    def __getitem__(self, key):
        if isinstance(key,basestring):
            return self._index[key]
        return self._count[key]

    def __setitem__(self, index, value):
        if isinstance(index, basestring):
            index = index.replace('(','[')
            index = index.replace(')',']')
        if type(value) is str:
           tmp = eval(value)
        else:
           tmp = value
        self._count[len(self._info)]=tmp
        self._info.append( (index, tmp) )
        self._index[index]=tmp
        #
        # If the index has the format x(1,a) or x[3,4,5]
        # then create a dictionary attribute 'x', which maps
        # the tuple values to the corresponding value.
        #
        if isinstance(index, basestring):
            if '[' in index:
                pieces = index.split('[')
                name = pieces[0]
                rest = None
                # when a variable is indexed by more than one parameter, you will
                # see identifiers of the form "x((a,b))" instead of the "x(a)"
                # one-dimensional form. this is due to the process of "stringifying"
                # the index, which is fine. it just requires a bit of ugliness in
                # the string splitting process.
                if index.count("]") == 2:
                   rest = pieces[2]
                else:
                   rest = pieces[1]
                # we're always taking the first part of the index,
                # so even in the two (or greater) dimensional case
                # such as "x((a,b))", the first piece is what we want.
                tpl = rest.split(']')[0]
                tokens = tpl.split(',')
                for i in xrange(len(tokens)):
                    tokens[i] = as_number(tokens[i])
                try:
                    var = getattr(self, name)
                except Exception, err:
                    setattr(self, name, {})
                    var = getattr(self, name)
                if len(tokens) == 1:
                    var[ tokens[0] ] = tmp
                else:
                    var[ tuple(tokens) ] = tmp
            else:
                setattr(self, index, tmp)
            
            

    def __len__(self):
        return len(self._info)

    def __iter__(self):
        return self._info.__iter__()

    def pprint(self, ostream=None, symbol_map=None):
        if ostream is None:
           ostream = sys.stdout
        if len(self._info) != 0:
           print >>ostream, "  "+self.description
           flag=True
           for (name,value) in self._info:
              # translate the name, if a map is available and the
              # item is in the map.
              if (symbol_map is not None) and (name in symbol_map):
                 name=symbol_map[name]
              if value != 0:
                 print >>ostream, "        "+str(name)+"\t"+format_io(value)
                 flag=False
           if flag:
              print >>ostream,   "        No nonzero values"


class Solution(object):

    def __init__(self):
        self.__dict__["_suffix"] = []
        self.value = None
        self.gap = None
        self.status = SolutionStatus.unsure
        self.add_suffix("variable","Primal Variables")
        self.add_suffix("dual","Dual Variables")
        self.add_suffix("slack","Constraint Slacks")
        self.add_suffix("constraint","Constraint Values")

    def __getattr__(self, name):
        try:
            if name in self.__dict__['_suffix']:
                return self.__dict__["_"+name]
        except KeyError:
            pass
        try:
           return self.__dict__[name]
        except KeyError:
           raise AttributeError, "Unknown attribute "+str(name)

    def __setattr__(self, name, val):
        if name in self._suffix:
           raise AttributeError, "Cannot set attribute '"+name+"' which is reserved for suffix array"
        self.__dict__[name]=val

    def add_suffix(self,name,description):
        self._suffix.append(name)
        setattr(self, "_"+name, _SolutionSuffix(description))

    def pprint(self, ostream=None, only_variables=False, symbol_map=None):
        if ostream is None:
           ostream = sys.stdout
        print >>ostream, "----------------------------------------------------------"
        print >>ostream, "------  Solution "+ str(self._counter+1)
        print >>ostream, "----------------------------------------------------------"
        tmp = self.__dict__.keys()
        tmp.sort()
        for key in tmp:
          if key[0] != "_":
             if key != "value":
                print >>ostream, " ",key+":", format_io(getattr(self, key))
             else:
                tmp = getattr(self,key)
                if isinstance(tmp, dict):
                   print >>ostream, "  Objectives"
                   for vkey in tmp:
                     print >>ostream, "        "+str(vkey)+"\t"+format_io(tmp[vkey])
                else:
                   print >>ostream, "  value:",format_io(tmp)
                   
        for item in self._suffix:
            if only_variables and item != "variable":
                continue
            getattr(self,"_"+item).pprint(ostream=ostream, symbol_map=symbol_map)


class SolutionSet(object):

    def __init__(self):
        self._solution={}
        self._counter=0
        self._ndx=[]

    def __len__(self):
        return len(self._solution)

    def __getitem__(self,i):
        return self._solution[i]

    def __call__(self,i=0):
        return self._solution[self._ndx[i]]

    def create(self):
        tmp = Solution()
        self.insert(tmp)
        return tmp

    def insert(self,solution):
        solution._counter = self._counter
        self._solution[self._counter] = solution
        self._ndx.append(self._counter)
        self._counter += 1

    def delete(self,i):
        del self._solution[i]
        self._ndx.remove(i)


class _Data_Container(object):
    pass


class SolverResults(object):

    def __init__(self):
        self.solver = _Data_Container()
        self.problem = _Data_Container()
        self.solution = SolutionSet()
        self.initialize()
        self.symbol_map = None 

    def initialize(self):
        #
        # Standard solver information
        #
        self.solver.status=SolverStatus.ok
        self.solver.systime=None
        self.solver.usrtime=None
        #
        # Standard problem information
        #
        self.problem.name=None
        self.problem.sense=ProblemSense.unknown
        self.problem.num_variables=None
        self.problem.num_constraints=None
        self.problem.num_objectives=None
        self.problem.lower_bound=None
        self.problem.upper_bound=None

    def __str__(self):
        output = StringIO.StringIO()
        self.write(ostream=output, num=sys.maxint)
        return output.getvalue()
        
    def write(self, filename=None, num=1, ostream=None, times=True, only_variables=False):
        if ostream is None:
           ostream = sys.stdout
        if not filename is None:
           OUTPUT=open(filename,"w")
           self.write(num=num,ostream=OUTPUT,times=times)
           OUTPUT.close()
           return

        print >>ostream, "=========================================================="
        print >>ostream, "---  Solver Results                                    ---"
        print >>ostream, "=========================================================="
        print >>ostream, "----------------------------------------------------------"
        print >>ostream, "------  Problem Information                         ------"
        print >>ostream, "----------------------------------------------------------"
        flag=False
        tmp = self.problem.__dict__.keys()
        tmp.sort()
        for key in tmp:
          if key[0] != "_":
             print >>ostream, " ",key+":", format_io(getattr(self.problem, key))
             flag=True
        if not flag:        # pragma:nocover
           print >>ostream, "  No Info"
        if self.symbol_map is not None:
           print >>ostream, "  Symbol map is available"
        #
        print >>ostream, "----------------------------------------------------------"
        print >>ostream, "------  Solver Information                          ------"
        print >>ostream, "----------------------------------------------------------"
        flag=False
        tmp = self.solver.__dict__.keys()
        tmp.sort()
        for key in tmp:
          if key[0] != "_":
             if times or not key in ["systime","usrtime"]:
                print >>ostream, " ",key+":", format_io(getattr(self.solver, key))
             flag=True
        if not flag:        # pragma:nocover
           print >>ostream, "  No Info"
        #
        if not num is None:
           print >>ostream, "  num_solutions: "+str(len(self.solution))
           i=0
           while i<min(num, len(self.solution)):
             self.solution(i).pprint(ostream, only_variables=only_variables, symbol_map=self.symbol_map)
             i += 1
        else:
           print >>ostream, "  num_solutions to display: 0"
        print >>ostream, "----------------------------------------------------------"

    def read(self, filename=None):          #pragma:nocover
        raise IOError, "SolverResults.read is not defined!"




#
# Unit Tests for coopr.opt.base.OS
#
#

import os
import sys
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__)))+"/../..")
cooprdir = dirname(abspath(__file__))+os.sep+".."+os.sep+".."+os.sep
currdir = dirname(abspath(__file__))+os.sep

from nose.tools import nottest
import coopr.opt
import coopr
import xml
import pyutilib.th as unittest
import pyutilib.services

pyutilib.services.TempfileManager.tempdir = currdir

class Test(unittest.TestCase):

    def setUp(self):
        pyutilib.services.TempfileManager.tempdir = currdir
        #
        # Create OSrL object
        #
        self.osrl = coopr.opt.reader.OS.OSrL()
        #
        # Initialize header info
        #
        self.osrl.header.serviceURI=""
        self.osrl.header.serviceName=""
        self.osrl.header.instanceName=""
        self.osrl.header.time=0.0
        self.osrl.header.message=""
        self.osrl.header.jobID=0

    def tearDown(self):
        pyutilib.services.TempfileManager.clear_tempfiles()

    def test_write_solution(self):
        #
        # Initialize solutions
        #
        self.osrl.numVariables=4
        self.osrl.numObjectives=2
        self.osrl.numConstraints=4
        self.soln = self.osrl.add_solution()
        self.soln.status_description=""
        self.soln.variable[0]=1.2
        self.soln.variable[1]=1.2
        self.soln.variable[3]=1.2
        self.soln.objective[-1]=1.2
        self.soln.objective[-2]=1.2
        self.soln.dual[0]=1.2
        self.soln.dual[1]=1.2
        self.soln.dual[3]=1.2
        self.osrl.write(currdir+"write_solution.OSrL.xml")
        if not os.path.exists(currdir+"write_solution.OSrL.xml"):
           self.fail("test_write_solution - failed to write OSrL.xml")
        self.failUnlessFileEqualsBaseline(currdir+"write_solution.OSrL.xml",currdir+"test1.OSrL.xml")

    def test_None_values(self):
        self.osrl.read(currdir+"test1.OSrL.xml")
        self.osrl.header.serviceName=None
        self.osrl.header.serviceURI=None
        self.osrl.header.instanceName=None
        self.osrl.header.time=None
        self.osrl.header.message=None
        self.osrl.header.jobID=None
        OUTPUT = open(currdir+"None_values.OSrL.xml","w")
        self.osrl.write(OUTPUT)
        OUTPUT.close()
        if not os.path.exists(currdir+"None_values.OSrL.xml"):
           self.fail("test_write_solution - failed to write OSrL.xml")
        self.failUnlessFileEqualsBaseline(currdir+"None_values.OSrL.xml",currdir+"test7.OSrL.xml")

    def test_read_solution(self):
        self.osrl.read(currdir+"test1.OSrL.xml")
        self.failUnlessEqual(len(self.osrl),1)
        
    def test_bad_status(self):
        self.osrl.header.generalStatus=None
        try:
          self.osrl.validate()
          self.fail("test_bad_status - Error validating 'None' status.")
        except ValueError:
          pass

    def test_test2_error(self):
        try:
          self.osrl.read(currdir+"test2.OSrL.xml")
          self.fail("test_test2_error - Failed to find error in test2.OSrL.xml")
        except ValueError:
          pass

    def test_test3_error(self):
        try:
          self.osrl.read(currdir+"test3.OSrL.xml")
          self.fail("test_test3_error - Failed to find error in test3.OSrL.xml")
        except KeyError:
          pass

    def test_test4_error(self):
        try:
          self.osrl.read(currdir+"test4.OSrL.xml")
          self.fail("test_test4_error - Failed to find error in test4.OSrL.xml")
        except KeyError:
          pass

    def test_test5_error(self):
        try:
          self.osrl.read(currdir+"test5.OSrL.xml")
          self.fail("test_test5_error - Failed to find error in test5.OSrL.xml")
        except KeyError:
          pass

    def test_test6_error(self):
        try:
          self.osrl.read(currdir+"test6.OSrL.xml")
          self.fail("test_test6_error - Failed to find error in test6.OSrL.xml")
        except xml.parsers.expat.ExpatError:
          pass
        try:
          self.osrl.read(currdir+"test6.OSrL")
          self.fail("test_test6_error - Failed raise an error for a missing file")
        except IOError:
          pass

    def test_factory(self):
        reader = coopr.opt.ReaderFactory("osrl")
        soln = reader(currdir+"test1.OSrL.xml")
        soln.write(filename=currdir+"test_os.txt")
        #
        # Compare with baseline
        #
        self.failUnlessFileEqualsBaseline(currdir+"test_os.txt", currdir+"test1.txt")
        

if __name__ == "__main__":
   unittest.main()



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['SolverInformation', 'SolverStatus', 'TerminationCondition']

from container import *
from pyutilib.enum import Enum


# Semantics: A coarse summary of how the solver terminated.
SolverStatus = Enum(
    'ok',                   # Normal termination
    'warning',              # Termination with unusual condition
    'error',                # Terminated internally with error
    'aborted',              # Terminated due to external conditions (e.g. interrupts)
    'unknown'               # An unitialized value 
    )

TerminationCondition = Enum(
    # OK
    'maxIterations',        # Exceeded maximum number of iterations allowed by user
    'minFunctionValue',     # Found solution smaller than specified function value
    'minStepLength',        # Step length is smaller than specified limit
    'globallyOptimal',      # Found a globally optimal solution
    'locallyOptimal',       # Found a locally optimal solution
    'optimal',              # Found an optimal solution
    'maxEvaluations',       # Exceeded maximum number of problem evaluations
    # WARNING
    'unbounded',            # Demonstrated that problem is unbounded
    'infeasible',           # Demonstrated that the problem is infeasible
    'invalidProlem',        # The problem setup or characteristics are not valide for the solver
    'other'                 # Other, uncategorized normal termination
    # ERROR
    'solverFailure',        # Solver failed to terminate correctly
    'internalSolverError',  # Internal solver error
    'error',                # Other errors
    # ABORTED
    'userInterrupt',        # Interrupt signal generated by user
    'resourceInterrupt',    # Interrupt signal in resources used by optimizer
    'licensingProblems',    # Problem accessing solver license
    # UNKNOWN
    'unknown'               # An unitialized value 
    )



class BranchAndBoundStats(MapContainer):

    def __init__(self):
        MapContainer.__init__(self)
        self.declare('number of bounded subproblems')
        self.declare('number of created subproblems')


class BlackBoxStats(MapContainer):

    def __init__(self):
        MapContainer.__init__(self)
        self.declare('number of function evaluations')
        self.declare('number of gradient evaluations')
        self.declare('number of iterations')


class SolverStatistics(MapContainer):

    def __init__(self):
        MapContainer.__init__(self)
        self.declare("branch_and_bound", value=BranchAndBoundStats(), active=False)
        self.declare("black_box", value=BlackBoxStats(), active=False)


class SolverInformation(MapContainer):

    def __init__(self):
        MapContainer.__init__(self)
        self.declare('name')
        self.declare('status', value=SolverStatus.ok)
        self.declare('return_code') # Semantics: the integer return code from the shell in which the solver is launched.
        self.declare('message')
        self.declare('user_time', type=ScalarType.time)
        self.declare('system_time', type=ScalarType.time)
        self.declare('wallclock_time', type=ScalarType.time)
        self.declare('termination_condition', value=TerminationCondition.unknown) # Semantics: The specific condition that caused the solver to terminate.
        self.declare('termination_message') # Semantics: A string pritned by the solver that summarizes the termination status.
        self.declare('statistics', value=SolverStatistics(), active=False)




#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________

__all__ = ['ProblemInformation', 'ProblemSense']

from container import *
from pyutilib.enum import Enum

ProblemSense = Enum('unknown', 'minimize', 'maximize')


class ProblemInformation(MapContainer):

    def __init__(self):
        MapContainer.__init__(self)
        self.declare('name')
        self.declare('lower_bound', value=float('-inf'))
        self.declare('upper_bound', value=float('inf'))
        self.declare('number_of_objectives', value=1, required=True)
        self.declare('number_of_constraints', value=0)
        self.declare('number_of_variables', value=0)
        self.declare('number_of_binary_variables')
        self.declare('number_of_integer_variables')
        self.declare('number_of_continuous_variables')
        self.declare('number_of_nonzeros')
        self.declare('sense', value=ProblemSense.unknown, required=True)
        



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['IProblemConverter', 'convert_problem']

import copy
import os
from formats import ProblemFormat, guess_format
from error import *
from pyutilib.component.core import *


class IProblemConverter(Interface):

    def can_convert(self, from_type, to_type):
        """Returns true if this object supports the specified conversion"""

    def convert(self, from_type, to_type):
        """Convert an instance of one type into another"""

    def apply(self, *args, **kwargs):
        """Convert an instance of one type into another"""


def convert_problem( args, target_problem_type, valid_problem_types,
                     has_capability=lambda x: False ):
    """
    Convert a problem, defined by the 'args' tuple, into another problem.
    """
    #print "HERE",args,target_problem_type,valid_problem_types
    if len(valid_problem_types) == 0:
       raise ConverterError, "No valid problem types"

    if not (target_problem_type is None or \
             target_problem_type in valid_problem_types):
        msg = "Problem type '%s' is not valid"
        raise ConverterError, msg % str( target_problem_type )

    if len(args) == 0:
       raise ConverterError, "Empty argument list"

    #
    # Setup list of source problem types
    #
    tmp = args[0]
    if isinstance(tmp,basestring):
       fname = tmp.split(os.sep)[-1]
       if os.sep in fname:   #pragma:nocover
          fname = tmp.split(os.sep)[-1]
       source_ptype = [guess_format(fname)]
       if source_ptype is [None]:
            raise ConverterError, "Unknown suffix type: "+suffix
    else:
       source_ptype = args[0].valid_problem_types()

    #
    # Setup list of valid problem types
    #
    valid_ptypes = copy.copy(valid_problem_types)
    if target_problem_type is not None:
       valid_ptypes.remove(target_problem_type)
       valid_ptypes = [target_problem_type]  + valid_ptypes
    if source_ptype[0] in valid_ptypes:
       ##print "HERE",source_ptype, valid_ptypes
       valid_ptypes.remove(source_ptype[0])
       valid_ptypes = [source_ptype[0]]  + valid_ptypes

    #
    # Iterate over the valid problem types, starting with the target type
    #
    # Apply conversion and return for first match
    #
    for ptype in valid_ptypes:
      for s_ptype in source_ptype:
        #print "HERE",str(s_ptype),str(ptype),args[0]
        #
        # If the source and target types are equal, then simply the return
        # the args (return just the first element of the tuple if it has length
        # one.
        #
        if s_ptype == ptype:
           return (args,ptype,None)
        #
        # Otherwise, try to convert
        #
        for converter in ExtensionPoint(IProblemConverter):
            #print "HERE",converter,s_ptype,ptype
            if converter.can_convert(s_ptype,ptype):
                tmp = [s_ptype,ptype] + list(args)
                tmp = tuple(tmp)
                tmpkw = {'capabilities':has_capability}
                problem_files, symbol_map = converter.apply(*tmp, **tmpkw)
                return problem_files, ptype, symbol_map

    msg = 'No conversion possible.  Source problem type: %s.  Valid target '  \
          'types: %s'
    raise ConverterError, msg % (str(source_ptype[0]), map(str,valid_ptypes))



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['SystemCallSolver']

import os
import sys
import time

from coopr.opt.base import *
from coopr.opt.base.solver import *
from coopr.opt.results import SolverStatus, SolverResults

from pyutilib.common import ApplicationError, WindowsError
from pyutilib.component.core import PluginGlobals
from pyutilib.misc import Bunch
from pyutilib.services import registered_executable, TempfileManager
from pyutilib.subprocess import run


class SystemCallSolver(OptSolver):
    """ A generic command line solver """

    def __init__(self, **kwargs):
        """ Constructor """
        OptSolver.__init__(self, **kwargs)
        self.keepFiles  = kwargs.pop('keepFiles', False )
        self.soln_file  = None
        self.log_file   = None
        self._timelimit = None
        self._timer     = ''


    def available(self, exception_flag=True):
        """ True if the solver is available """
        if self._assert_available:
            return True
        if not OptSolver.available(self,exception_flag):
            return False
        ans=self.executable()
        if ans is None:
           if exception_flag:
              if self.executable() is None:
                  msg = "No executable found for solver '%s'"
                  raise ApplicationError, msg % self.name
              else:
                  msg = "Bad path for application 'PICO' used by solver %s: " \
                       '%s'
                  raise ApplicationError, msg % ( self.name, self.executable())
           return False
        return True

    def create_command_line(self,executable,problem_files):
        """
        Create the command line that is executed.
        """
        raise NotImplementedError       #pragma:nocover

    def process_logfile(self):
        """
        Process the logfile for information about the optimization process.
        """
        return SolverResults()

    def process_soln_file(self,results):
        """
        Process auxilliary data files generated by the optimizer (e.g. solution
        files)
        """
        return results

    def _executable(self):
        """
        Returns the executable used by this solver.
        """
        raise NotImplementedError

    def _presolve(self, *args, **kwds):
        """
        Peform presolves.
        """
        if 'keepFiles' in kwds:
            self.keepFiles = kwds['keepFiles']
            del kwds['keepFiles']
        OptSolver._presolve(self, *args, **kwds)
        #
        # Verify that the input problem exists
        #
        for filename in self._problem_files:
            if not os.path.exists(filename):
                msg = 'Solver failed to locate input problem file: %s'
                raise ValueError, msg % filename
        #
        # Create command line
        #
        self._command = self.create_command_line(
                              self.executable(), self._problem_files )
        self.log_file=self._command.log_file
        #
        # The pre-cleanup is probably unncessary, but also not harmful.
        #
        if self.log_file is not None and os.path.exists(self.log_file):
           os.remove(self.log_file)
        if self.soln_file is not None and os.path.exists(self.soln_file):
           os.remove(self.soln_file)


    def _apply_solver(self):
        if registered_executable('timer'):
            self._timer = registered_executable('timer').get_path()
        #
        # Execute the command
        #
        PluginGlobals.env("coopr.opt").log.debug("Running "+self._command.cmd)

        # display the log/solver file names prior to execution. this is useful
        # in case something crashes unexpectedly, which is not without precedent.
        if self.keepFiles:
           if self.log_file is not None:
              print "Solver log file: '%s'" % self.log_file
           if self.soln_file is not None:
              print "Solver solution file: '%s'" % self.soln_file
           if self._problem_files is not []:
              print "Solver problem files: %s" % str(self._problem_files)

        sys.stdout.flush()
        [rc,log] = self._execute_command(self._command)
        sys.stdout.flush()
        self._status = Bunch(rc=rc, log=log)

    def _postsolve(self):

        if self.log_file is not None:
           OUTPUT=open(self.log_file,"w")
           print >>OUTPUT, "Solver command line: "+self._command.cmd
           print >>OUTPUT, ""
           print >>OUTPUT, self._status.log
           OUTPUT.close()

        # JPW: The cleanup of the problem file probably shouldn't be here, but
        #   rather in the base OptSolver class. That would require movement of
        #   the keepFiles attribute and associated cleanup logic to the base
        #   class, which I didn't feel like doing at this present time. the
        #   base class remove_files method should clean up the problem file.

        if self.log_file is not None and not os.path.exists(self.log_file):
            msg = "File '%s' not generated while executing %s"
            raise IOError, msg % ( self.log_file, self.path )
        results = None
        if self._results_format is not None:
           results = self.process_output(self._status.rc)
           if not self.keepFiles:
              TempfileManager.clear_tempfiles()
              # in some cases, the solution filename is not generated via the temp-file mechanism,
              # instead being automatically derived from the input lp/nl filename. so, we may
              # have to clean it up manually.
              if not self.soln_file is None and os.path.exists(self.soln_file):
                os.remove(self.soln_file)

        return results

    def _execute_command(self,command):
        """
        Execute the command
        """
        try:
            [rc, log] = run(
                command.cmd,
                timelimit = self._timelimit,
                env   = command.env,
                tee   = self.tee,
                shell = True
             )
        except WindowsError, err:
            msg = 'Could not execute the command: %s\tError message: %s'
            raise ApplicationError, msg % ( command.cmd, str(err) )
        sys.stdout.flush()
        return [rc,log]

    def process_output(self,rc):
        """
        Process the output files.
        """
        start_time = time.time()
        if self._results_format is None:
           raise ValueError, "Results format is None"
        results = self.process_logfile()
        log_file_completion_time = time.time()
        if self._report_timing is True:
           print "Log file read time="+str(log_file_completion_time - start_time)+" seconds"
        if self.results_reader is None:
           self.process_soln_file(results)
           soln_file_completion_time = time.time()
           if self._report_timing is True:
              print "Solution file read time="+str(soln_file_completion_time - log_file_completion_time)+" seconds"
        else:
           results = self.results_reader(self.results_file, res=results)
           results_reader_completion_time = time.time()
           if self._report_timing is True:
                print "Results reader time="+str(results_reader_completion_time - log_file_completion_time)+" seconds"
        if rc != None:
           results.solver.error_rc=rc
           if rc != 0:
              results.solver.status=SolverStatus.error
        return results

    def _default_results_format(self, prob_format):
        """ Returns the default results format for different problem
            formats.
        """
        return ResultsFormat.soln





#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________

"""
Script to generate the installer for coopr.opt.
"""

import glob
import os

def _find_packages(path):
    """
    Generate a list of nested packages
    """
    pkg_list=[]
    if not os.path.exists(path):
        return []
    if not os.path.exists(path+os.sep+"__init__.py"):
        return []
    else:
        pkg_list.append(path)
    for root, dirs, files in os.walk(path, topdown=True):
      if root in pkg_list and "__init__.py" in files:
         for name in dirs:
           if os.path.exists(root+os.sep+name+os.sep+"__init__.py"):
              pkg_list.append(root+os.sep+name)
    return map(lambda x:x.replace(os.sep,"."), pkg_list)

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

from setuptools import setup
packages = _find_packages('coopr')

scripts = glob.glob("scripts/*")

setup(name='coopr.opt',
      version='2.4',
      maintainer='William E. Hart',
      maintainer_email='wehart@sandia.gov',
      url = 'https://software.sandia.gov/svn/public/coopr/coopr.opt',
      license = 'BSD',
      platforms = ["any"],
      description = 'Coopr generic optimization interfaces',
      long_description = read('README.txt'),
      classifiers = [
            'Development Status :: 4 - Beta',
            'Intended Audience :: End Users/Desktop',
            'Intended Audience :: Science/Research',
            'License :: OSI Approved :: BSD License',
            'Natural Language :: English',
            'Operating System :: Microsoft :: Windows',
            'Operating System :: Unix',
            'Programming Language :: Python',
            'Programming Language :: Unix Shell',
            'Topic :: Scientific/Engineering :: Mathematics',
            'Topic :: Software Development :: Libraries :: Python Modules',
            'Topic :: Scientific/Engineering'
        ],
      packages=packages,
      keywords=['optimization'],
      scripts=scripts,
      namespace_packages=['coopr']
      )




#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

#
# Utility classes for dealing with the COIN-OR optimization
# services
#

import os
import sys
from xml.dom import minidom, Node
import xml
from pyutilib.enum import Enum
from pyutilib.misc import get_xml_text
from coopr.opt import SolutionStatus

#
# A class for reading/writing OSrL file
#
class OSrL(object):

    GeneralStatus = Enum('error', 'warning', 'success')
    SolutionStatus = Enum('unbounded', 'globallyOptimal', 'locallyOptimal', 
                        'optimal', 'bestSoFar', 'feasible', 'infeasible', 
                        'stoppedByLimit', 'unsure', 'error', 'other')

    class ResultHeader(object):
        def __init__(self):
            self.generalStatus=OSrL.GeneralStatus.error
            self.serviceURI=None
            self.serviceName=None
            self.instanceName=None
            self.time=None
            self.message=None
            self.jobID=None

        def _parse_xml(self, node):
            for child in node.childNodes:
              if child.nodeType == Node.ELEMENT_NODE:
                 if child.nodeName in ["generalStatus","serviceURI","serviceName","instanceName","time","message", "jobID"]:
                    setattr(self,child.nodeName, get_xml_text(child))

        def validate(self):
            if self.generalStatus not in OSrL.GeneralStatus:
               raise ValueError, "General status value '"+str(self.generalStatus)+"' is not in OSrL.GeneralStatus"

        #def __repr__(self):
            #return "OSrL ResultHeader"
            
        #__str__ = __repr__

        def write(self,ostream=None, prefix=""):
            if ostream is None:
               ostream = sys.stdout
            self.validate()
            os.write(ostream.fileno(), prefix)
            os.write(ostream.fileno(), "<ResultHeader>\n")
            #
            os.write(ostream.fileno(), prefix+"  <generalStatus>")
            os.write(ostream.fileno(), str(self.generalStatus))
            os.write(ostream.fileno(), "</generalStatus>\n")
            #
            if self.serviceURI is not None:
               os.write(ostream.fileno(), prefix+"  <serviceURI>"+str(self.serviceURI)+"</serviceURI>\n")
            #
            os.write(ostream.fileno(), prefix+"  <serviceName>")
            if self.serviceName is None:
               os.write(ostream.fileno(), "unknown")
            else:
               os.write(ostream.fileno(), str(self.serviceName))
            os.write(ostream.fileno(), "</serviceName>\n")
            #
            os.write(ostream.fileno(), prefix+"  <instanceName>")
            if self.instanceName is None:
               os.write(ostream.fileno(), "unknown")
            else:
               os.write(ostream.fileno(), str(self.instanceName))
            os.write(ostream.fileno(), "</instanceName>\n")
            #
            if self.time is not None:
               os.write(ostream.fileno(), prefix+"  <time>"+str(self.time)+"</time>\n")
            #
            if self.message is not None:
               os.write(ostream.fileno(), prefix+"  <message>"+str(self.message)+"</message>\n")
            #
            if self.jobID is not None:
               os.write(ostream.fileno(), prefix+"  <jobID>"+str(self.jobID)+"</jobID>\n")
            #
            os.write(ostream.fileno(), prefix)
            os.write(ostream.fileno(), "</ResultHeader>\n")


    class Solution(object):
        def __init__(self):
            self.status_type=OSrL.SolutionStatus.unsure
            self.status_description=None
            self.message=None
            self.variable={}
            self.dual={}
            self.objective={}
            self.other={}

        def _parse_xml(self, node):
            for child in node.childNodes:
              if child.nodeType == Node.ELEMENT_NODE:
                 if child.nodeName.lower() == "message":
                    self.message = get_xml_text(child)

                 elif child.nodeName.lower() == "status":
                    for (name,value) in child.attributes.items():
                      if name == "type":
                             self.status_type = str(value)
                      elif name == "description":
                             self.status_description = str(value)
                 elif child.nodeName.lower() == "variables":
                    for gchild in child.childNodes:
                      if gchild.nodeType == Node.ELEMENT_NODE and\
                         gchild.nodeName.lower() == "values":
                         for ggchild in gchild.childNodes:
                           if ggchild.nodeType == Node.ELEMENT_NODE and\
                              ggchild.nodeName.lower() == "var":
                              index=None
                              for (name,value) in ggchild.attributes.items():
                                if name == "idx":
                                   index = int(value)
                              self.variable[index] = get_xml_text(ggchild)
                 elif child.nodeName.lower() == "objectives":
                    for gchild in child.childNodes:
                      if gchild.nodeType == Node.ELEMENT_NODE and\
                         gchild.nodeName.lower() == "values":
                         for ggchild in gchild.childNodes:
                           if ggchild.nodeType == Node.ELEMENT_NODE and\
                              ggchild.nodeName.lower() == "obj":
                              index=None
                              for (name,value) in ggchild.attributes.items():
                                  if name == "idx":
                                   index = int(value)
                              self.objective[index] = get_xml_text(ggchild)
                 elif child.nodeName.lower() == "constraints":
                    for gchild in child.childNodes:
                      if gchild.nodeType == Node.ELEMENT_NODE and\
                         gchild.nodeName.lower() == "dualvalues":
                         for ggchild in gchild.childNodes:
                           if ggchild.nodeType == Node.ELEMENT_NODE and\
                              ggchild.nodeName.lower() == "con":
                              index=None
                              for (name,value) in ggchild.attributes.items():
                                  if name == "idx":
                                   index = int(value)
                              self.dual[index] = get_xml_text(ggchild)

        def validate(self):
            if self.status_type not in OSrL.SolutionStatus:
               raise ValueError, "Solution status value '"+str(self.status_type)+"' is not in OSrL.SolutionStatus"
            for key in self.variable:
              if type(key) is not int or (type(key) is int and key < 0):
                 raise KeyError, "Solution variables index '"+str(key)+"' is not an nonnegative integer"
            for key in self.dual:
              if type(key) is not int or (type(key) is int and key < 0):
                 raise KeyError, "Solution constraint index '"+str(key)+"' is not an nonnegative integer"
            for key in self.objective:
              if type(key) is not int or (type(key) is int and key >= 0):
                 raise KeyError, "Solution objective index '"+str(key)+"' is not a negative integer"

        def write(self,ostream=None, prefix=""):
            if ostream is None:
               ostream = sys.stdout
            self.validate()
            os.write(ostream.fileno(), prefix)
            os.write(ostream.fileno(), "<solution>\n")
            #
            os.write(ostream.fileno(), prefix+"  <status type=\""+str(self.status_type)+"\"")
            if self.status_description is not None:
               os.write(ostream.fileno(), " description=\""+str(self.status_description)+"\"")
            os.write(ostream.fileno(), " />\n")
            #
            if len(self.variable) > 0:
               os.write(ostream.fileno(), prefix+"  <variables>\n")
               os.write(ostream.fileno(), prefix+"    <values>\n")
               tmp = self.variable.keys()
               tmp.sort()
               for ndx in tmp:
                 os.write(ostream.fileno(), prefix+"      <var idx=\""+str(ndx)+"\">"+str(self.variable[ndx])+"</var>\n")
               os.write(ostream.fileno(), prefix+"    </values>\n")
               os.write(ostream.fileno(), prefix+"  </variables>\n")
            #
            if len(self.objective) > 0:
               os.write(ostream.fileno(), prefix+"  <objectives>\n")
               os.write(ostream.fileno(), prefix+"    <values>\n")
               tmp = self.objective.keys()
               tmp.sort()
               for ndx in tmp:
                 os.write(ostream.fileno(), prefix+"      <obj idx=\""+str(ndx)+"\">"+str(self.objective[ndx])+"</obj>\n")
               os.write(ostream.fileno(), prefix+"    </values>\n")
               os.write(ostream.fileno(), prefix+"  </objectives>\n")
            #
            if len(self.dual) > 0:
               os.write(ostream.fileno(), prefix+"  <constraints>\n")
               os.write(ostream.fileno(), prefix+"    <dualValues>\n")
               tmp = self.dual.keys()
               tmp.sort()
               for ndx in tmp:
                 os.write(ostream.fileno(), prefix+"      <con idx=\""+str(ndx)+"\">"+str(self.dual[ndx])+"</con>\n")
               os.write(ostream.fileno(), prefix+"    </dualValues>\n")
               os.write(ostream.fileno(), prefix+"  </constraints>\n")
            #
            os.write(ostream.fileno(), prefix)
            os.write(ostream.fileno(), "</solution>\n")


    def __init__(self):
        self.header = OSrL.ResultHeader()
        self.numVariables=0
        self.numConstraints=0
        self.numObjectives=0
        self.solution = []

    def add_solution(self):
        tmp = OSrL.Solution()
        self.solution.append(tmp)
        return tmp

    def __len__(self):
        return len(self.solution)

    def __iter__(self):
        return self.solution.__iter__()

    def validate(self):
        self.header.validate()
        for soln in self.solution:
          soln.validate()

    def write(self,ostream=None, prefix=""):
        if ostream is None:
           ostream = sys.stdout
        self.validate()
        if type(ostream) is str:
           self._tmpfile = ostream
           ostream = open(self._tmpfile,"w")
        else:
           self._tmpfile = None

        os.write(ostream.fileno(), prefix)
        os.write(ostream.fileno(), "<OSrL>\n")

        self.header.write(ostream,prefix+"  ")

        os.write(ostream.fileno(), prefix+"  <ResultData>\n")
        os.write(ostream.fileno(), prefix+"    <optimization\n")
        os.write(ostream.fileno(), prefix+"      numberOfSolutions=\""+str(len(self.solution))+"\"\n")
        os.write(ostream.fileno(), prefix+"      numberOfVariables=\""+str(self.numVariables)+"\"\n")
        os.write(ostream.fileno(), prefix+"      numberOfObjectives=\""+str(self.numObjectives)+"\"\n")
        os.write(ostream.fileno(), prefix+"      numberOfConstraints=\""+str(self.numConstraints)+"\"\n")
        os.write(ostream.fileno(), prefix+"    >\n")

        for soln in self.solution:
          soln.write(ostream, prefix+"      ")

        os.write(ostream.fileno(), prefix+"    </optimization>\n")
        os.write(ostream.fileno(), prefix+"  </ResultData>\n")

        os.write(ostream.fileno(), prefix)
        os.write(ostream.fileno(), "</OSrL>\n")
        if self._tmpfile is not None:
           ostream.close()

    def read(self, filename):
        try:
          doc = minidom.parse(filename)
        except xml.parsers.expat.ExpatError, e:
             raise xml.parsers.expat.ExpatError, "Problem parsing XML file " + filename+": "+str(e)
        except Exception, e:
             raise
        try:
          self._parse_xml(doc.documentElement)
        except:                                 #pragma:nocover
          raise IOError, "Problem initializing with data from the XML file " + filename
        self.validate()

    def _parse_xml(self, node):
        for child in node.childNodes:
          if child.nodeType == Node.ELEMENT_NODE and \
             child.nodeName.lower() == "resultheader":
             self.header._parse_xml(child)
          if child.nodeType == Node.ELEMENT_NODE and \
             child.nodeName.lower() == "resultdata":
             for gchild in child.childNodes:
               if gchild.nodeType == Node.ELEMENT_NODE and \
                  gchild.nodeName.lower() == "optimization":
                  for (name,value) in gchild.attributes.items():
                      if name == "numberOfSolutions":
                            self.numSolutions = str(value)
                      elif name == "numberOfObjectives":
                            self.numObjectives = str(value)
                      elif name == "numberOfVariables":
                            self.numVariables = str(value)
                      elif name == "numberOfConstraints":
                            self.numConstraints = str(value)
                  for ggchild in gchild.childNodes:
                    if ggchild.nodeType == Node.ELEMENT_NODE and \
                       ggchild.nodeName.lower() == "solution":
                       tmp = self.add_solution()
                       tmp._parse_xml(ggchild)




from coopr.opt.base import results
from coopr.opt.base.formats import *
from coopr.opt import SolverResults
from pyutilib.component.core import alias

class ResultsReader_osrl(results.AbstractResultsReader):
    """
    Class that reads in an OSrL results file and generates a 
    SolverResults object.
    """

    alias(str(ResultsFormat.osrl))

    def __init__(self, name=None):
        results.AbstractResultsReader.__init__(self,ResultsFormat.osrl)
        if not name is None:
            self.name = name

    def __call__(self, filename, res=None):
        if res is None:
            res = SolverResults()  
        osrl = OSrL()
        osrl.read(filename)
        #
        # Transfer solver info into the SolverResults object
        #  
        ##res.solver.status = SolverResults.SolverStatus(osrl.generalStatus)
        res.solver.serviceURI = osrl.header.serviceURI
        res.solver.serviceName = osrl.header.serviceName
        res.solver.instanceName = osrl.header.instanceName
        res.solver.systime = osrl.header.time
        res.solver.message = osrl.header.message
        res.solver.id = osrl.header.jobID
        res.problem.name = osrl.header.instanceName
        #
        # Transfer solution info into the SolverResults object
        #  
        for soln in osrl:
          #
          # We create a solution, and then populate its data
          # The solution object is managed by the SolverResults instance
          #
          solution = res.solution.add()
          solution.status = SolutionStatus(soln.status_type)
          solution.status_description = soln.status_description
          solution.message = soln.message
          for key in soln.variable:
            solution.variable[key+1] = soln.variable[key]
          for key in soln.dual:
            solution.constraint[key+1].dual = soln.dual[key]
          i=0
          for key in soln.objective:
            solution.objective['f'+str(i)] = soln.objective[key]
            solution.objective['f'+str(i)].id = key
            i=i+1
          solution.other=soln.other
          res.problem.number_of_constraints = len(soln.dual) 
          res.problem.number_of_variables = len(soln.variable) 
          res.problem.number_of_objectives = len(soln.objective) 
        return res




#
# Unit Tests for coopr.opt.opt_config
#
#

import os
import sys
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__)))+"/../..")
cooprdir = dirname(abspath(__file__))+os.sep+".."+os.sep+".."+os.sep
currdir = dirname(abspath(__file__))+os.sep

import unittest
from nose.tools import nottest
import coopr.opt
import pyutilib.services
import pyutilib.component.app
import pyutilib.misc

pyutilib.services.TempfileManager.tempdir = currdir

class OptConfigDebug(unittest.TestCase):

    def setUp(self):
        pyutilib.services.TempfileManager.tempdir = currdir

    def tearDown(self):
        pyutilib.services.TempfileManager.clear_tempfiles()

    def test_config1(self):
        """
        Read in config file opt1.cfg
        """
        app = pyutilib.component.app.SimpleApplication("testapp")
        #pyutilib.component.PluginGlobals.pprint()
        #app.config.summarize()
        app.save_configuration(currdir+"opt1-out.cfg")
        app.configure(currdir+"opt1.cfg")
        if pyutilib.services.registered_executable("pico_convert"):
            self.failUnlessEqual( pyutilib.services.registered_executable("pico_convert").get_path(), pyutilib.misc.search_file("pico_convert"))
        if pyutilib.services.registered_executable("glpsol"):
            self.failUnlessEqual( pyutilib.services.registered_executable("glpsol").get_path(), pyutilib.misc.search_file("glpsol"))
        if pyutilib.services.registered_executable("ampl"):
            self.failUnlessEqual( pyutilib.services.registered_executable("ampl").get_path(), pyutilib.misc.search_file("ampl"))
        if pyutilib.services.registered_executable("timer"):
            self.failUnlessEqual( pyutilib.services.registered_executable("timer").get_path(), pyutilib.misc.search_file("timer"))


if __name__ == "__main__":
   unittest.main()
